// -*-c++-*-

/*
 * $Id: Particle.cpp,v 1.1.1.1 2001/12/29 18:28:07 uli Exp $
 *
 * Particle System
 */

#include "Particle.h"


/**********************************************************************
 *
 * Particle
 *
 **********************************************************************/

// Minimum particle mass
const float ParticleMinMass = 0.0005f;

// Constructor
Particle::Particle()
{
    _pos.set(0.0f, 0.0f, 0.0f);
    _vel.set(0.0f, 0.0f, 0.0f);
    _force.set(0.0f, 0.0f, 0.0f);
    setMass(1.0f);
    _lifeRemain = _lifeSpan = 0.0f;
}

// Set mass, store reciprocal value for faster divides
void Particle::setMass(float mass)
{
    if (mass < ParticleMinMass)
        mass = ParticleMinMass;
    _massRecip = 1.0f / mass;
}

// Compute new position
void Particle::update(float dt)
{
    if (_lifeRemain <= 0.0f)
        return;
    _lifeRemain -= dt;

    // Calculate acceleration, velocity, and position
    osg::Vec3 accel = _force * _massRecip;
    _vel += accel * dt;
    _pos += _vel * dt;

    // Clear force for next update()
    _force.set(0.0f, 0.0f, 0.0f);
}
