// -*-c++-*-

/*
 * $Id: Particle.h,v 1.1.1.1 2001/12/29 18:28:07 uli Exp $
 *
 * Particle
 */

#ifndef _PARTICLE_H_
#define _PARTICLE_H_

#include <vector>
#include <osg/Vec3>


/**
 * Particle class.
 */
class Particle {
public:
    /// Constructor.
    Particle();

    /// Destructor.
    virtual ~Particle() { }

    /// Position.
    osg::Vec3 _pos;

    /// Velocity.
    osg::Vec3 _vel;

    /// Force.
    osg::Vec3 _force;

    /// Remaining life-span.
    float _lifeRemain;

    /// Set life-span.
    void setLifeSpan(float lifeSpan) {
        _lifeRemain = _lifeSpan = lifeSpan;
    }

    /// Set mass.
    void setMass(float mass);

    /// Get mass.
    float getMass() const { return 1.0f / _massRecip; }

    /// Get normalized age [0..1] of the particle.
    float getAge() const {
        return (_lifeSpan != 0.0f ? 1.0f - (_lifeRemain / _lifeSpan) : 0.0f);
    }

    /**
     * Update particle.
     * This method updates the particle velocity and position according to
     * the forces acting upon the particle. Subclasses can define this
     * method for more control.
     */
    virtual void update(float dt = 1.0f);

    /**
     * Draw particles.
     * This method is called by ParticleSystem with the list and number of
     * particles. Subclasses must define this class and handle it accordingly.
     * @param particle STL-vector of Particle pointers. Only alive Particles
     * must be drawn, i.e. those with _lifeRemain > 0.0.
     * @param first Index of the first alive Particle.
     * @param last Index of the last alive Particle.
     */
    virtual void draw(vector<Particle*>* particle,
                      unsigned int first, unsigned int last) = 0;

protected:
    /// Total life-span.
    float _lifeSpan;

    /// Particle mass.
    float _massRecip;
};

#endif
