// -*-c++-*-

/*
 * $Id: ParticleAction.h,v 1.1.1.1 2001/12/29 18:28:07 uli Exp $
 *
 * Particle Action
 */

#ifndef _PARTICLEACTION_H_
#define _PARTICLEACTION_H_

#include <vector>
#include <osg/Vec3>


/// Particle class.
class Particle;


/**
 * ParticleAction class.
 *
 * A particle action defines a force acting on particles in a particle system.
 */
class ParticleAction {
public:
    /// Constructor.
    ParticleAction() { }

    /// Destructor.
    virtual ~ParticleAction() { }

    /**
     * Apply force to all 'alive' particles; calls 'applyForce'.
     * @param particle STL-vector of Particle pointers. Only alive Particles need
     * to be taken into account, i.e. those Particles with _lifeRemain > 0.
     * @param first Index of the first alive Particle.
     * @param last Index of the last alive Particle.
     */
    void apply(vector<Particle*>& particle, unsigned int first, unsigned int last);

protected:
    /// Apply force.
    virtual void applyForce(Particle* particle) = 0;
};

/**
 * GravityAction class.
 * "It isn't just a good idea -- it's the law."
 */
class GravityAction : public ParticleAction {
public:
    /// Constructor.
    GravityAction() : ParticleAction() { }

protected:
    /// Apply force.
    virtual void applyForce(Particle* particle);
};

extern GravityAction s_GravityAction;

#endif
