// -*-c++-*-

/*
 * $Id: ParticleSource.cpp,v 1.1.1.1 2001/12/29 18:28:07 uli Exp $
 *
 * Particle Source
 */

#include <stdlib.h>
#include <iostream.h>

#include "ParticleSource.h"
#include "ParticleSystem.h"


/**********************************************************************
 *
 * Particle Source
 *
 **********************************************************************/

// Constructor
ParticleSource::ParticleSource()
{
    _perSecond = 0;
    _pos.set(0.0f, 0.0f, 0.0f);
    _posVar.set(0.1f, 0.1f, 0.1f);
    _vel.set(0.0f, 0.0f, 1.0f);
    _velVar.set(1.0f, 1.0f, 1.0f);
    setLifeSpan(3.0f, 0.5f);
    setMass(1.0f, 0.0f);
}

// Emit particles
void ParticleSource::emit(vector<Particle*>& particle, float dt)
{
    unsigned int count = (unsigned int) ((float) _perSecond * dt);

    // Emit at least one particle
    count = MAX(count, 1);

    for (unsigned int i = 0; i < particle.size() && count; i++) {
        Particle* p = particle[i];
        if (p && p->_lifeRemain <= 0.0f) {
            emitParticle(p);
            count--;
        }
    }
}

// Default particle
void ParticleSource::emitParticle(Particle* particle)
{
    particle->_pos.set(variance(_pos[0], _posVar[0]),
                       variance(_pos[1], _posVar[1]),
                       variance(_pos[2], _posVar[2]));

    particle->_vel.set(variance(_vel[0], _velVar[0]),
                       variance(_vel[1], _velVar[1]),
                       variance(_vel[2], _velVar[2]));

    particle->setLifeSpan(variance(_lifeSpan, _lifeSpanVar));

    particle->setMass(variance(_mass, _massVar));
}

// Return base +/- variance
float ParticleSource::variance(float base, float var)
{
    float f = (var <= 0.0f ? base : base + (var * rand()) / (float) (RAND_MAX) - var * 0.5f);
    //cerr << "base=" << base << " var=" << var << " f=" << f << endl;
    return f;
}
