// -*-c++-*-

/*
 * $Id: ParticleSystem.h,v 1.2 2002/02/24 13:57:57 uli Exp $
 *
 * Particle System
 */

#ifndef _PARTICLESYSTEM_H_
#define _PARTICLESYSTEM_H_

#include "Particle.h"
#include "ParticleSource.h"
#include "ParticleAction.h"

#include <vector>
#include <osg/Vec3>
#include <osg/Drawable>
#include <osg/Geode>


// Macros
#ifndef MAX
#define MAX(a, b) ((a) > (b) ? (a) : (b))
#endif

#ifndef MIN
#define MIN(a, b) ((a) < (b) ? (a) : (b))
#endif

#ifndef ABS
#define ABS(a) ((a) < 0.0f ? -(a) : (a))
#endif


/**
 * Particle system drawable class.
 */
class ParticleSystem : public osg::Drawable {
public:
    /**
     * Constructor.
     * @param particle Particle pool. The particles in this pool are emitted by
     * the attached ParticleSources. They need not all be active at a time,
     * this depends on the rate of Particles that are emitted by the sources
     * per second.
     */
    ParticleSystem(vector<Particle*>* particle);

#if 0
    ParticleSystem(const ParticleSystem& that, const osg::CopyOp& op) {
        _particle = that._particle;
        _source = that._particle;
        _action = that._action;
        _firstAlive = that._firstAlive;
        _lastAlive = that._lastAlive;
    }
#endif

    /// Destructor.
    virtual ~ParticleSystem();

    // TODO: Cloning etc. need some work on constructor.
    virtual Object* cloneType() const {
        return new ParticleSystem(_particle);
    }

    /// Clone copy.
    virtual Object* clone(const osg::CopyOp& /*copyop*/) const {
        return NULL; //new ParticleSystem(*this, copyop);
    }

    /// Type information.
    virtual bool isSameKindAs(const osg::Object* obj) const {
        return dynamic_cast<const ParticleSystem*>(obj) != NULL;
    }

    /// Class name.
    virtual const char* className() const { return "ParticleSystem"; }

    /// Return statistics.
    virtual bool getStats(osg::Statistics&);

    /// Add ParticleSource.
    void addSource(ParticleSource* source);

    /// Add ParticleAction.
    void addAction(ParticleAction* action);

    /**
     * Update particles.
     * @param dt Time in seconds since the last frame. This value is used to
     * provide frame rate independent behaviour.
     */
    void update(float dt = 1.0f);

    /**
     * Draw particles.
     */
    virtual void drawImmediateMode(osg::State& state);

private:
    /// List of particles.
    vector<Particle*>* _particle;

    /// List of particle sources.
    vector<ParticleSource*> _source;

    /// List of particle actions.
    vector<ParticleAction*> _action;

    /// First 'alive' particle.
    unsigned int _firstAlive;

    /// Last 'alive' particle.
    unsigned int _lastAlive;

    /// Bounding box is computed in 'update'.
    virtual const bool computeBound() const { return true; }
};


/**
 * ParticleNode class.
 * Gives the ParticleSystem a place in the world.
 */
class ParticleNode : public osg::Geode {
public:
    /// Constructor.
    ParticleNode();
};

#endif
